#pragma once

namespace nstd {

	#pragma pack(push,1)
	struct RGBA { uchar r,g,b,a; };
	#pragma pack(pop)

	class Color {
	public:
		union { struct { int mRed,mGreen,mBlue,mAlpha; }; struct { int r,g,b,a; }; int m4[4]; };

		Color( int R=0, int G=0, int B=0, int A=255 ) : mRed(R), mGreen(G), mBlue(B), mAlpha(A) {}
		Color( uint32 C ) : mRed((C>>16)&0xFF), mGreen((C>>8)&0xFF), mBlue((C)&0xFF), mAlpha((C>>24)&0xFF) {}
		Color( uint32 C, int A ) : mRed((C>>16)&0xFF), mGreen((C>>8)&0xFF), mBlue((C)&0xFF), mAlpha(A) {}
		Color( RGBA const& c )  : mRed(c.r), mGreen(c.g), mBlue(c.b), mAlpha(c.a) {}
		Color( const uchar* theElements ) : mRed(theElements[0]), mGreen(theElements[1]), mBlue(theElements[2]), mAlpha(0xFF) {}
		Color( const int* theElements ) : mRed(theElements[0]), mGreen(theElements[1]), mBlue(theElements[2]), mAlpha(0xFF) {}

		static const uint32 WHITE   = 0xFFFFFFFF;
		static const uint32 BLACK   = 0xFF000000;
		static const uint32 BLUE    = 0xFF0000FF;
		static const uint32 GREEN   = 0xFF00FF00;
		static const uint32 RED     = 0xFFFF0000;
		static const uint32 YELLOW  = 0xFFFFFF00;
		static const uint32 MAGENTA = 0xFFFF00FF;
		static const uint32 AZURE   = 0xFF00FFFF;
		static const uint32 GREY    = 0xFF808080;

		uint32			toInt()   const { return ( mAlpha << 24 ) | ( mRed << 16 ) | ( mGreen << 8 ) | ( mBlue ); }
		uint32          toRGBA()  const { return ( mAlpha << 24 ) | ( mRed ) | ( mGreen << 8 ) | ( mBlue << 16 ); }
		
		int&			operator[]( int theIdx ) { return m4[theIdx]; }
		int				operator[]( int theIdx ) const { return m4[theIdx]; }

		friend Color operator * ( Color const& theColor, float coef );
		friend Color operator * ( Color const& C1, Color const& C2 );
		friend Color operator + ( Color const& C1, Color const& C2 );
		friend Color operator - ( Color const& C1, Color const& C2 );

		static uint32 PackedMul( uint32 const lhs, uint32 const rhs );
		
		static uint32 ToRGBA( uint32 argb ) { return (argb&0xFF00FF00)|((argb>>16)&0xFF)|((argb<<16)&0xFF0000); }
		
		Color&  operator += ( Color const& theColor ) { *this=*this+theColor; return *this; }
		Color&  operator - () { mRed=-mRed; mGreen=-mGreen; mBlue=-mBlue; mAlpha=-mAlpha; return *this; }
		bool    operator ==( Color const& rhs ) const { return r==rhs.r && g==rhs.g && b==rhs.b && a==rhs.a; }
		bool    operator !=( Color const& rhs ) const { return r!=rhs.r || g!=rhs.g || b!=rhs.b || a!=rhs.a; }
	};
}

#if 0

namespace argo { namespace math {

	template<> Agon::Color GetLerp< Agon::Color, Agon::Color >( Agon::Color const& C0, Agon::Color const& C1, float const t ) { 
		const int r = GetLerp< int >( C0.mRed,   C1.mRed, t );
		const int g = GetLerp< int >( C0.mGreen, C1.mGreen, t );
		const int b = GetLerp< int >( C0.mBlue,  C1.mBlue, t );
		const int a = GetLerp< int >( C0.mAlpha, C1.mAlpha, t );
		return Agon::Color( r, g, b, a );
	}

}}

namespace Agon {

Color operator * ( const Color& theColor, float coef ) {
	Color c;
	c.mRed   = Median<int>(0,argo::math::Float2Int(theColor.mRed*coef),255);
	c.mGreen = Median<int>(0,argo::math::Float2Int(theColor.mGreen*coef),255);
	c.mBlue  = Median<int>(0,argo::math::Float2Int(theColor.mBlue*coef),255);
	c.mAlpha = Median<int>(0,argo::math::Float2Int(theColor.mAlpha*coef),255);
	return c;
}

Color operator + ( const Color& C1, const Color& C2 ) {
	Color c;
	c.mRed   = Median<int>(0,C1.mRed+C2.mRed,255);
	c.mGreen = Median<int>(0,C1.mGreen+C2.mGreen,255);
	c.mBlue  = Median<int>(0,C1.mBlue+C2.mBlue,255);
	c.mAlpha = Median<int>(0,C1.mAlpha+C2.mAlpha,255);

	return c;
}

Color operator * ( const Color& C1, const Color& C2 ) {
	int r = C1.mRed   * C2.mRed;
	int g = C1.mGreen * C2.mGreen;
	int b = C1.mBlue  * C2.mBlue;
	int a = C1.mAlpha * C2.mAlpha;
	r = ( r + ( r>>8 ) + 255 )>>8;
	g = ( g + ( g>>8 ) + 255 )>>8;
	b = ( b + ( b>>8 ) + 255 )>>8;
	a = ( a + ( a>>8 ) + 255 )>>8;
	return Color( r, g, b, a );
}

Color operator - ( const Color& C1, const Color& C2 ) {
	Color c;
	c.mRed   = Median<int>(0,C1.mRed-C2.mRed,255);
	c.mGreen = Median<int>(0,C1.mGreen-C2.mGreen,255);
	c.mBlue  = Median<int>(0,C1.mBlue-C2.mBlue,255);
	c.mAlpha = Median<int>(0,C1.mAlpha-C2.mAlpha,255);
	return c;
}

uint32 Color::PackedMul( uint32 const lhs, uint32 const rhs ) {
	uint32 const c1 = ( ( ( lhs>> 0 )&0x0000FF ) * ((rhs>>0)&0xFF)  + 0x0000FF ) >> 8;
	uint32 const c2 = ( ( ( lhs>> 8 )&0x0000FF ) * ((rhs>>8)&0xFF) + 0x0000FF ) & 0x0000FF00;
	uint32 const c3 = ( ( ( lhs>> 8 )&0x00FF00 ) * ((rhs>>16)&0xFF)   + 0x00FF00 ) & 0x00FF0000;
	uint32 const c4 = ( ( ( lhs>> 8 )&0xFF0000 ) * ((rhs>>24)) + 0xFF0000 ) & 0xFF000000;
	return c1 | c2 | c3 | c4;
}
#endif



