#pragma once
#include "nstd_prefix.h"
#include <boost/intrusive_ptr.hpp>
#include <boost/atomic.hpp>

namespace nstd {

class RefCounted {
public:
    RefCounted() : refcount_(0) {}
    RefCounted( RefCounted const& )  : refcount_(0) {}
    void operator=( RefCounted const& ) {}

    void addref() { ++refcount_; }
    void subref() { if( 0== --refcount_ ) deleteThis(); }
    int refs() const { return refcount_; }
    void subrefNotDelete() { --refcount_; ASSERT(0==refcount_); }
	
protected:
    virtual ~RefCounted() { ASSERT(refcount_==0); }
    virtual void deleteThis() { delete this; }
private:
    int refcount_;
};

class RefCountedAtomic {
public:
    RefCountedAtomic() : refcount_(0) {}
    RefCountedAtomic( RefCountedAtomic const& )  : refcount_(0) {}
    void operator=( RefCountedAtomic const& ) {}

    void addref() { refcount_.fetch_add(1, boost::memory_order_relaxed); }
    void subref() { 
		if( refcount_.fetch_sub(1, boost::memory_order_release) == 1 ) {
			boost::atomic_thread_fence(boost::memory_order_acquire);
			deleteThis();
		}
	}
    int refs() const { return refcount_; }
    void subrefNotDelete() { --refcount_; ASSERT(0==refcount_); }
	
protected:
    virtual ~RefCountedAtomic() { ASSERT(refcount_==0); }
    virtual void deleteThis() { delete this; }
private:
    mutable boost::atomic<int> refcount_;
};


/*
class GCAutoGuard {
	GCAutoGuard( GCAutoGuard const& ); void operator=( GCAutoGuard const& ); /// noncopyable
public:
	GCCount& mValue;
	GCAutoGuard( GCCount& theValue ) : mValue( theValue ) { theValue.addref(); }
	~GCAutoGuard() { mValue.subrefNotDelete(); }
};

template < typename T > class GCAuto {
	T mValue;
public:
	GCAuto() { mValue.addref(); }
	template < typename TP1 > GCAuto( TP1 const& tp1 ) : mValue( tp1 ) { mValue.addref(); }
	template < typename TP1, typename TP2 > GCAuto( TP1 const& tp1, TP2 const& tp2 ) : mValue( tp1, tp2 ) { mValue.addref(); }
	template < typename TP1, typename TP2, typename TP3 > GCAuto( TP1 const& tp1, TP2 const& tp2, TP3 const& tp3 ) : mValue( tp1, tp2, tp3 ) { mValue.addref(); }

	~GCAuto() { mValue.subrefNotDelete(); }

	T* operator&() { return &mValue; }
	T const* operator&() const { return &mValue; }
	T* operator->() { return &mValue; }
	T const* operator->() const { return &mValue; }
};
*/

}

namespace boost {
	inline void intrusive_ptr_add_ref( nstd::RefCounted* p ) { p->addref(); }
	inline void intrusive_ptr_release( nstd::RefCounted* p ) { p->subref(); }
	inline void intrusive_ptr_add_ref( nstd::RefCountedAtomic* p ) { p->addref(); }
	inline void intrusive_ptr_release( nstd::RefCountedAtomic* p ) { p->subref(); }
}

/*
namespace nstd {
	struct DLinkedHookRefCounted {
		static void DLinkedUnlinkHook( RefCounted* refs, nstd::DLinkedBaseClass const* ) { refs->subref(); }
		static void DLinkedInsertHook( RefCounted* refs, nstd::DLinkedBaseClass const* node ) { ASSERT(!node->inList()); refs->addref(); }
	};
	using boost::intrusive_ptr_add_ref;
	using boost::intrusive_ptr_release;
}
*/

